@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,28 +24,22 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import os
+import os, glib
 
 homedir = os.path.expanduser("~")
 lastdir = homedir
 
-data_home = os.getenv("XDG_DATA_HOME")
-if data_home == None:
-    data_home = os.path.join(homedir, ".local", "share")
+data_home = glib.get_user_data_dir()
 data_home = os.path.join(data_home, "exaile")
 if not os.path.exists(data_home):
     os.makedirs(data_home)
 
-config_home = os.getenv("XDG_CONFIG_HOME")
-if config_home == None:
-    config_home = os.path.join(homedir, ".config")
+config_home = glib.get_user_config_dir()
 config_home = os.path.join(config_home, "exaile")
 if not os.path.exists(config_home):
     os.makedirs(config_home)
 
-cache_home = os.getenv("XDG_CACHE_HOME")
-if cache_home == None:
-    cache_home = os.path.join(homedir, ".cache")
+cache_home = glib.get_user_cache_dir()
 cache_home = os.path.join(cache_home, "exaile")
 if not os.path.exists(cache_home):
     os.makedirs(cache_home)
@@ -79,7 +73,7 @@ def get_config_dir():
     return config_home
 
 def get_config_dirs():
-    return config_dirs
+    return config_dirs[:]
 
 def get_data_dir():
     return data_home
@@ -90,21 +84,24 @@ def get_data_dirs():
 def get_cache_dir():
     return cache_home
 
-def get_data_path(*subpath_elements):
+
+def _get_path(basedirs, *subpath_elements, **kwargs):
+    check_exists = kwargs.get("check_exists", True)
     subpath = os.path.join(*subpath_elements)
-    for d in data_dirs:
+    for d in basedirs:
         path = os.path.join(d, subpath)
-        if os.path.exists(path):
+        if not check_exists or os.path.exists(path):
             return path
     return None
 
-def get_config_path(*subpath_elements):
-    subpath = os.path.join(*subpath_elements)
-    for d in config_dirs:
-        path = os.path.join(d, subpath)
-        if os.path.exists(path):
-            return path
-    return None
+def get_data_path(*subpath_elements, **kwargs):
+    return _get_path(data_dirs, *subpath_elements, **kwargs)
+
+def get_config_path(*subpath_elements, **kwargs):
+    return _get_path(config_dirs, *subpath_elements, **kwargs)
+
+def get_data_home_path(*subpath_elements, **kwargs):
+    return _get_path([data_home], *subpath_elements, **kwargs)
 
 def get_last_dir():
     return lastdir
