@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,12 +30,15 @@ import logging
 import os
 import urlparse
 
-import gobject
+import gio
+import glib
 import gtk
 
 from xl.nls import gettext as _
 logger = logging.getLogger(__name__)
 from xl import xdg, common, event, metadata, settings, playlist as _xpl
+from xl import covers
+
 try:
     import gtk.glade
     gtk.glade.textdomain('exaile')
@@ -46,20 +49,14 @@ except ImportError:
         "Failed to import gtk.glade, interface "
         "will not be fully translated.")
 
-from xlgui import commondialogs, cover
-from xlgui import devices, guiutil, icons, prefs, queue
-
-
-###
-# Set up xl/event to work with the gtk event loop
-logger.info("Setting up deferred idle manager function...")
-event.IDLE_MANAGER._call_function = guiutil.idle_add()(
-    event.IDLE_MANAGER._call_function)
+from xlgui import cover
+from xlgui import devices, guiutil, icons, preferences, queue
+from xlgui.widgets import dialogs
 
 def mainloop():
     gtk.main()
 
-def controller():
+def get_controller():
     return Main._main
 
 class Main(object):
@@ -76,6 +73,8 @@ class Main(object):
         from xlgui import main, panel, tray, progress
         from xlgui.panel import collection, radio, playlists, files
 
+        gtk.gdk.set_program_class("Exaile")
+
         self.exaile = exaile
         self.first_removed = False
         self.tray_icon = None
@@ -85,23 +84,22 @@ class Main(object):
         self.progress_box = self.builder.get_object('progress_box')
         self.progress_manager = progress.ProgressManager(self.progress_box)
 
-        self.icons = icons.IconManager()
-        self.icons.add_icon_name_from_directory('exaile',
+        icons.MANAGER.add_icon_name_from_directory('exaile',
             xdg.get_data_path('images'))
         gtk.window_set_default_icon_name('exaile')
-        self.icons.add_icon_name_from_directory('exaile-pause',
+        icons.MANAGER.add_icon_name_from_directory('exaile-pause',
             xdg.get_data_path('images'))
-        self.icons.add_icon_name_from_directory('exaile-play',
+        icons.MANAGER.add_icon_name_from_directory('exaile-play',
             xdg.get_data_path('images'))
 
         for name in ('dynamic', 'repeat', 'shuffle'):
             icon_name = 'media-playlist-%s' % name
-            self.icons.add_icon_name_from_directory(icon_name,
+            icons.MANAGER.add_icon_name_from_directory(icon_name,
                 xdg.get_data_path('images'))
 
         logger.info("Loading main window...")
         self.main = main.MainWindow(self, self.builder,
-            exaile.collection, exaile.player, exaile.queue, exaile.covers)
+            exaile.collection, exaile.player, exaile.queue, covers.MANAGER)
         self.panel_notebook = self.builder.get_object('panel_notebook')
         self.play_toolbar = self.builder.get_object('play_toolbar')
 
@@ -140,7 +138,7 @@ class Main(object):
         pl = self.main.get_current_playlist ().playlist
         name = pl.get_name() + ".m3u"
 
-        dialog = commondialogs.FileOperationDialog(_("Export current playlist..."),
+        dialog = dialogs.FileOperationDialog(_("Export Current Playlist"),
             None, gtk.FILE_CHOOSER_ACTION_SAVE,
             buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
             gtk.STOCK_SAVE, gtk.RESPONSE_OK))
@@ -163,18 +161,27 @@ class Main(object):
                 try:
                     _xpl.export_playlist(pl, path)
                 except _xpl.InvalidPlaylistTypeException:
-                    commondialogs.error(None, _('Invalid file extension, file not saved'))
+                    dialogs.error(None, _('Invalid file extension, file not saved'))
         dialog.destroy()
 
     def open_url(self, *e):
         """
             Displays a dialog to open a url
         """
-        dialog = commondialogs.TextEntryDialog(_('Enter the URL to open'),
+        dialog = dialogs.TextEntryDialog(_('Enter the URL to open'),
         _('Open URL'))
         dialog.set_transient_for(self.main.window)
         dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
 
+        clipboard = gtk.clipboard_get()
+        text = clipboard.wait_for_text()
+
+        if text is not None:
+            location = gio.File(uri=text)
+
+            if location.get_uri_scheme() is not None:
+                dialog.set_value(text)
+
         result = dialog.run()
         dialog.hide()
         if result == gtk.RESPONSE_OK:
@@ -279,14 +286,14 @@ class Main(object):
         """
             Shows the cover manager
         """
-        window = cover.CoverManager(self.main.window, self.exaile.covers,
+        window = cover.CoverManager(self.main.window,
             self.exaile.collection)
 
     def show_preferences(self):
         """
             Shows the preferences dialog
         """
-        dialog = prefs.PreferencesDialog(self.main.window, self)
+        dialog = preferences.PreferencesDialog(self.main.window, self)
         dialog.run()
 
     def show_devices(self):
@@ -355,12 +362,20 @@ class Main(object):
         """
             Called when the user wishes to rescan the collection
         """
-        if not self.exaile.collection._scanning:
-            from xlgui.collection import CollectionScanThread
-            thread = CollectionScanThread(self, self.exaile.collection,
-                    self.panels['collection'])
+        libraries = self.exaile.collection.get_libraries()
+        if not self.exaile.collection._scanning and len(libraries) > 0:
+            from xl.collection import CollectionScanThread
+
+            thread = CollectionScanThread(self.exaile.collection)
+            thread.connect('done', self.on_rescan_done)
             self.progress_manager.add_monitor(thread,
-                _("Scanning collection..."), 'gtk-refresh')
+                _("Scanning collection..."), gtk.STOCK_REFRESH)
+
+    def on_rescan_done(self, thread):
+        """
+            Called when the rescan has finished
+        """
+        glib.idle_add(self.panels['collection'].load_tree)
 
     def on_randomize_playlist(self, *e):
         pl = self.main.get_selected_playlist()
@@ -441,6 +456,7 @@ class Main(object):
 
     @guiutil.idle_add()
     def add_device_panel(self, type, obj, device):
+        from xl.collection import CollectionScanThread
         from xlgui.panel.device import DevicePanel, FlatPlaylistDevicePanel
         import xlgui.panel
 
@@ -459,13 +475,15 @@ class Main(object):
             self.main.on_append_items(items, sort=sort))
         panel.connect('queue-items', lambda panel, items, sort=sort:
             self.main.on_append_items(items, queue=True, sort=sort))
+        panel.connect('replace-items', lambda panel, items, sort=sort:
+            self.main.on_append_items(items, replace=True, sort=sort))
 
         self.device_panels[device.get_name()] = panel
-        gobject.idle_add(self.add_panel, *panel.get_panel())
-        thread = collection.CollectionScanThread(self.main,
-                device.get_collection(), panel)
+        glib.idle_add(self.add_panel, *panel.get_panel())
+        thread = CollectionScanThread(device.get_collection())
+        thread.connect('done', panel.load_tree)
         self.progress_manager.add_monitor(thread,
-                _("Scanning %s..."%device.name), 'gtk-refresh')
+            _("Scanning %s..." % device.name), gtk.STOCK_REFRESH)
 
     @guiutil.idle_add()
     def remove_device_panel(self, type, obj, device):
@@ -483,15 +501,17 @@ def show_splash(show=True):
         @param show: [bool] show the splash screen
     """
     if not show: return
-    image = gtk.Image()
-    image.set_from_file(xdg.get_data_path("images/splash.png"))
     builder = gtk.Builder()
     builder.add_from_file(xdg.get_data_path("ui/splash.ui"))
+    image = builder.get_object('splash_image')
+    image.set_from_file(xdg.get_data_path("images/splash.png"))
     splash_screen = builder.get_object('SplashScreen')
-    box = builder.get_object('splash_box')
-    box.pack_start(image, True, True)
     splash_screen.set_transient_for(None)
+
+    # Show the splash screen without causing startup notification.
+    gtk.window_set_auto_startup_notification(False)
     splash_screen.show_all()
+    gtk.window_set_auto_startup_notification(True)
 
     #ensure that the splash gets completely drawn before we move on
     while gtk.events_pending():
