@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,14 +31,15 @@ import urllib
 
 import gio
 import glib
-import gtk
 import gobject
+import gtk
 
 from xl import common, trax, metadata
 from xl import settings
 from xl import event
-from xlgui import panel, guiutil, xdg, menu, playlist
+from xlgui import guiutil, panel, playlist, menu, xdg
 from xl.nls import gettext as _
+import xlgui
 
 class FilesPanel(panel.Panel):
     """
@@ -46,6 +47,7 @@ class FilesPanel(panel.Panel):
     """
     __gsignals__ = {
         'append-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+        'replace-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
         'queue-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
     }
 
@@ -66,12 +68,16 @@ class FilesPanel(panel.Panel):
         self._setup_widgets()
         self.menu = menu.FilesPanelMenu()
         self.menu.connect('append-items', lambda *e:
-            self.emit('append-items', self.get_selected_tracks()))
+            self.emit('append-items', self.tree.get_selected_tracks()))
+        self.menu.connect('replace-items', lambda *e:
+            self.emit('replace-items', self.tree.get_selected_tracks()))
         self.menu.connect('queue-items', lambda *e:
-            self.emit('queue-items', self.get_selected_tracks()))
-        self.menu.connect('rating-set', self.set_rating)
+            self.emit('queue-items', self.tree.get_selected_tracks()))
+        self.menu.connect('rating-changed', self.set_rating)
         self.menu.connect('properties', lambda *e:
             self.properties_dialog())
+        self.menu.connect('view-items', lambda *e:
+            self._on_view_items())
 
         self.key_id = None
         self.i = 0
@@ -86,20 +92,27 @@ class FilesPanel(panel.Panel):
             Shows the properties dialog
         """
         from xlgui import properties
-        tracks = self.get_selected_tracks()
+        tracks = self.tree.get_selected_tracks()
 
         if not tracks:
             return False
 
         tracks_sorted = trax.sort_tracks(
-			('artist', 'date', 'album', 'discnumber', 'tracknumber'),
-			tracks)
+            ('artist', 'date', 'album', 'discnumber', 'tracknumber'),
+            tracks)
 
         dialog = properties.TrackPropertiesDialog(self.parent,
             tracks_sorted)
 
+    def _on_view_items(self, *args):
+        """
+            Opens a file manager in the containing directory
+        """
+        track = self.tree.get_selected_tracks()[0]
+        common.open_file_directory(track.get_loc_for_io())
+
     def set_rating(self, widget, rating):
-        tracks = self.get_selected_tracks()
+        tracks = self.tree.get_selected_tracks()
         for track in tracks:
             track.set_rating(rating)
 
@@ -108,7 +121,7 @@ class FilesPanel(panel.Panel):
             Sets up tree widget for the files panel
         """
         self.model = gtk.ListStore(gio.File, gtk.gdk.Pixbuf, str, str)
-        self.tree = tree = guiutil.DragTreeView(self, True, True)
+        self.tree = tree = FilesDragTreeView(self, True, True)
         tree.set_model(self.model)
         tree.connect('row-activated', self.row_activated)
         tree.connect('key-release-event', self.on_key_released)
@@ -280,7 +293,7 @@ class FilesPanel(panel.Panel):
             if ftype == gio.FILE_TYPE_DIRECTORY:
                 self.load_directory(f)
             else:
-                self.emit('append-items', self.get_selected_tracks())
+                self.emit('append-items', self.tree.get_selected_tracks())
 
     def key_release(self, *e):
         """
@@ -288,10 +301,10 @@ class FilesPanel(panel.Panel):
             Sets up a timer to simulate live-search
         """
         if self.key_id:
-            gobject.source_remove(self.key_id)
+            glib.source_remove(self.key_id)
             self.key_id = None
 
-        self.key_id = gobject.timeout_add(700, lambda *e:
+        self.key_id = glib.timeout_add(700, lambda *e:
             self.load_directory(self.current, history=False,
             keyword=unicode(self.search.get_text(), 'utf-8')))
 
@@ -434,16 +447,43 @@ class FilesPanel(panel.Panel):
                 self.forward.set_sensitive(False)
             self.up.set_sensitive(bool(directory.get_parent()))
 
-        gobject.idle_add(idle)
+        glib.idle_add(idle)
 
+    def drag_data_received(self, *e):
+        """
+            stub
+        """
+        pass
+
+    def drag_data_delete(self, *e):
+        """
+            stub
+        """
+        pass
+
+    def drag_get_data(self, treeview, context, selection, target_id, etime):
+        """
+            Called when a drag source wants data for this drag operation
+        """
+        tracks = self.tree.get_selected_tracks()
+        if not tracks: return
+        for track in tracks:
+            guiutil.DragTreeView.dragged_data[track.get_loc_for_io()] = track
+        uris = trax.util.get_uris_from_tracks(tracks)
+        selection.set_uris(uris)
+
+class FilesDragTreeView(guiutil.DragTreeView):
+    """
+        Custom DragTreeView to retrieve data from files
+    """
     def get_selected_tracks(self):
         """
-            Returns the selected tracks
+            Returns the currently selected tracks
         """
-        selection = self.tree.get_selection()
+        selection = self.get_selection()
         model, paths = selection.get_selected_rows()
-
         tracks = []
+
         for path in paths:
             f = model[path][0]
             self.append_recursive(tracks, f)
@@ -475,25 +515,3 @@ class FilesPanel(panel.Panel):
         tr = trax.Track(uri)
         return tr
 
-    def drag_data_received(self, *e):
-        """
-            stub
-        """
-        pass
-
-    def drag_data_delete(self, *e):
-        """
-            stub
-        """
-        pass
-
-    def drag_get_data(self, treeview, context, selection, target_id, etime):
-        """
-            Called when a drag source wants data for this drag operation
-        """
-        tracks = self.get_selected_tracks()
-        if not tracks: return
-        for track in tracks:
-            guiutil.DragTreeView.dragged_data[track.get_loc_for_io()] = track
-        urls = guiutil.get_urls_for(tracks)
-        selection.set_uris(urls)
