@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,15 +28,27 @@ import os.path
 import time
 import urllib
 
+import glib
 import gobject
 import gtk
 
-from xlgui import panel, guiutil, xdg, commondialogs
-from xlgui import menu, filtergui
-from xlgui import playlist as guiplaylist
-from xl import playlist, settings, event
-from xlgui.filtergui import MultiEntryField, EntryField
+from xl import (
+    event,
+    playlist,
+    radio,
+    settings,
+    trax
+)
 from xl.nls import gettext as _
+from xlgui import (
+    guiutil,
+    menu,
+    panel,
+    playlist as guiplaylist,
+    xdg
+)
+from xlgui.widgets import dialogs
+from xlgui.widgets.filter import *
 
 class EntrySecondsField(MultiEntryField):
     def __init__(self):
@@ -53,22 +65,22 @@ class EntryDaysField(MultiEntryField):
 
 DATE_FIELDS = [
     _('seconds'), _('minutes'), _('hours'), _('days'), _('weeks')]
-class SpinDateField(filtergui.SpinButtonAndComboField):
+class SpinDateField(SpinButtonAndComboField):
     def __init__(self):
-        filtergui.SpinButtonAndComboField.__init__(self, DATE_FIELDS)
+        SpinButtonAndComboField.__init__(self, DATE_FIELDS)
 
-class SpinSecondsField(filtergui.SpinLabelField):
+class SpinSecondsField(SpinLabelField):
     def __init__(self):
-        filtergui.SpinLabelField.__init__(self, _('seconds'))
+        SpinLabelField.__init__(self, _('seconds'))
 
-class SpinRating(filtergui.SpinLabelField):
+class SpinRating(SpinLabelField):
     def __init__(self):
-        filtergui.SpinLabelField.__init__(self, '', 
-                settings.get_option("miscellaneous/rating_steps", 5), 0)
+        SpinLabelField.__init__(self, '', 
+                settings.get_option("rating/maximum", 5), 0)
 
-class SpinNothing(filtergui.SpinLabelField):
+class SpinNothing(SpinLabelField):
     def __init__(self):
-        filtergui.SpinLabelField.__init__(self, '')
+        SpinLabelField.__init__(self, '')
 
 CRITERIA = [
     (_('Artist'), [
@@ -124,20 +136,16 @@ CRITERIA = [
         (_('at most'), SpinSecondsField),
         (_('is'), SpinSecondsField),
     ]),
-#    (_('Date Added'), [
+    (_('Date added'), [
         # TRANSLATORS: Example: track has been added in the last 2 days
-#        (_('in the last'), (SpinDateField,
-#            lambda x, i: day_calc(x, i, 'time_added'))),
+        (_('in the last'), SpinDateField),
         # TRANSLATORS: Example: track has not been added in the last 5 hours
-#        (_('not in the last'), (SpinDateField,
-#            lambda x, i: day_calc(x, i, 'time_added', '<'))),
-#        ]),
-#    (_('Last Played'), [
-#        (_('in the last'), (SpinDateField,
-#            lambda x, i: day_calc(x, i, 'last_played'))),
-#        (_('not in the last'), (SpinDateField,
-#            lambda x, i: day_calc(x, i, 'last_played', '<'))),
-#        ]),
+        (_('not in the last'), SpinDateField),
+    ]),
+    (_('Last played'), [
+        (_('in the last'), SpinDateField),
+        (_('not in the last'), SpinDateField),
+    ]),
     (_('Location'), [
         (_('is'), EntryField),
         (_('is not'), EntryField),
@@ -158,6 +166,8 @@ _TRANS = {
     _('between'): '><',
     _('greater than'): '>',
     _('less than'): '<',
+    _('in the last'): '>=',
+    _('not in the last'): '<',
 }
 
 _NMAP = {
@@ -169,6 +179,8 @@ _NMAP = {
     _('Plays'): '__playcount',
     _('Year'): 'date',
     _('Genre'): 'genre',
+    _('Date added'): '__date_added',
+    _('Last played'): '__last_played',
     _('Location'): '__loc',
 }
 
@@ -197,6 +209,7 @@ class BasePlaylistPanelMixin(gobject.GObject):
         'playlist-selected': (gobject.SIGNAL_RUN_LAST, None, (object,)),
         'tracks-selected': (gobject.SIGNAL_RUN_LAST, None, (object,)),
         'append-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
+        'replace-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
         'queue-items': (gobject.SIGNAL_RUN_LAST, None, (object,)),
     }
     def __init__(self):
@@ -213,7 +226,7 @@ class BasePlaylistPanelMixin(gobject.GObject):
             Removes the selected playlist from the UI
             and from the underlying manager
         """
-        selected_playlist = self.get_selected_playlist(raw=True)
+        selected_playlist = self.tree.get_selected_playlist(raw=True)
         if selected_playlist is not None:
             if isinstance(selected_playlist, playlist.SmartPlaylist):
                 self.smart_manager.remove_playlist(
@@ -234,81 +247,35 @@ class BasePlaylistPanelMixin(gobject.GObject):
         """
         if name in self.playlist_manager.playlists:
             # name is already in use
-            commondialogs.error(self.parent, _("The "
+            dialogs.error(self.parent, _("The "
                 "playlist name you entered is already in use."))
             return
 
-        pl = self.get_selected_playlist()
-        if pl is not None:
-            old_name = pl.get_name()
+        playlist = self.tree.get_selected_playlist()
+        if playlist is not None:
+            old_name = playlist.get_name()
             selection = self.tree.get_selection()
             (model, iter) = selection.get_selected()
             model.set_value(iter, 1, name)
             #Update the manager aswell
-            self.playlist_manager.rename_playlist(pl, name)
+            self.playlist_manager.rename_playlist(playlist, name)
 
     def open_selected_playlist(self):
         selection = self.tree.get_selection()
         (model, iter) = selection.get_selected()
         self.open_item(self.tree, model.get_path(iter), None)
 
-    def get_selected_playlist(self, raw=False):
+    def on_rating_changed(self, widget, rating):
         """
-            Retrieves the currently selected playlist in
-            the playlists panel.  If a non-playlist is
-            selected it returns None
-
-            @return: the playlist
+            Updates the rating of the selected tracks
         """
-        item = self.get_selected_item(raw=raw)
-        if isinstance(item, (playlist.Playlist,
-            playlist.SmartPlaylist)):
-            return item
-        else:
-            return None
-
-    def get_selected_track(self):
-        item = self.get_selected_item()
-        if not item: return None
-        if isinstance(item, TrackWrapper):
-            return item.track
-        else:
-            return None
-
-    def get_selected_item(self, raw=False):
-        selection = self.tree.get_selection()
-        (model, iter) = selection.get_selected()
-        if not iter: return None
-        item = model.get_value(iter, 2)
-        # for smart playlists
-        if isinstance(item, playlist.SmartPlaylist):
-            if raw: return item
-            return item.get_playlist(self.collection)
-        elif isinstance(item, playlist.Playlist) :
-            return item
-        elif isinstance(item, TrackWrapper):
-            return item
-        else:
-            return None
-
-    def get_selected_tracks(self):
-        """
-            Used by the menu, just basically gets the selected
-            playlist and returns the tracks in it
-        """
-        pl = self.get_selected_playlist()
-        if pl is not None:
-            return pl.get_tracks()
-        else:
-            return self.get_selected_track()
-
-        return None
-
-    def set_rating(self, widget, rating):
         tracks = self.get_selected_tracks()
-        steps = settings.get_option('miscellaneous/rating_steps', 5)
+
         for track in tracks:
-            track['__rating'] = float((100.0*rating)/steps)
+            track.set_rating(rating)
+
+        maximum = settings.get_option('rating/maximum', 5)
+        event.log_event('rating_changed', self, rating / maximum * 100)
 
     def open_item(self, tree, path, col):
         """
@@ -335,54 +302,98 @@ class BasePlaylistPanelMixin(gobject.GObject):
             else:
                 self.emit('append-items', [item.track])
 
-    def add_new_playlist(self, tracks = [], name = None):
+    def add_new_playlist(self, tracks=[], name = None):
         do_add_playlist = False
         if name:
             if name in self.playlist_manager.playlists:
                 # name is already in use
-                commondialogs.error(self.parent, _("The "
+                dialogs.error(self.parent, _("The "
                     "playlist name you entered is already in use."))
             else:
                 do_add_playlist = True
         else:
             if tracks:
-                artist = tracks[0].get_tag_raw('artist')
-                composer = tracks[0].get_tag_raw('composer')
-                album = tracks[0].get_tag_raw('album')
-                sameartist = True
-                samecomposer = True
-                samealbum = True
+                artists = []
+                composers = []
+                albums = []
 
                 for track in tracks:
-                    if artist != track.get_tag_raw('artist'):
-                        sameartist = False
+                    artist = track.get_tag_display('artist',
+                        artist_compilations=False)
 
-                    if composer != track.get_tag_raw('composer'):
-                        samecomposer = False
+                    if artist is not None:
+                        artists += [artist]
 
-                    if album != track.get_tag_raw('album'):
-                        samealbum = False
+                    composer = track.get_tag_display('composer',
+                        artist_compilations=False)
 
-                if sameartist:
-                    name = tracks[0].get_tag_display('artist', artist_compilations=False)
-                elif samecomposer and composer:
-                    name = tracks[0].get_tag_display('composer', artist_compilations=False)
-                elif samealbum:
-                    name = tracks[0].get_tag_display('album', artist_compilations=False)
+                    if composer is not None:
+                        composers += composer
 
-            dialog = commondialogs.TextEntryDialog(
+                    album = track.get_tag_display('album')
+
+                    if album is not None:
+                        albums += album
+
+                artists = list(set(artists))[:3]
+                composers = list(set(composers))[:3]
+                albums = list(set(albums))[:3]
+
+                if len(artists) > 0:
+                    name = artists[0]
+
+                    if len(artists) > 2:
+                        # TRANSLATORS: Playlist title suggestion with more than two values
+                        name = _('%(first)s, %(second)s and others') % {
+                            'first': artists[0], 'second': artists[1]
+                        }
+                    elif len(artists) > 1:
+                        # TRANSLATORS: Playlist title suggestion with two values
+                        name = _('%(first)s and %(second)s') % {
+                            'first': artists[0], 'second': artists[1]
+                        }
+                elif len(composers) > 0:
+                    name = composers[0]
+
+                    if len(composers) > 2:
+                        # TRANSLATORS: Playlist title suggestion with more than two values
+                        name = _('%(first)s, %(second)s and others') % {
+                            'first': composers[0], 'second': composers[1]
+                        }
+                    elif len(composers) > 1:
+                        # TRANSLATORS: Playlist title suggestion with two values
+                        name = _('%(first)s and %(second)s') % {
+                            'first': composers[0], 'second': composers[1]
+                        }
+                elif len(albums) > 0:
+                    name = albums[0]
+
+                    if len(albums) > 2:
+                        # TRANSLATORS: Playlist title suggestion with more than two values
+                        name = _('%(first)s, %(second)s and others') % {
+                            'first': albums[0], 'second': albums[1]
+                        }
+                    elif len(albums) > 1:
+                        # TRANSLATORS: Playlist title suggestion with two values
+                        name = _('%(first)s and %(second)s') % {
+                            'first': albums[0], 'second': albums[1]
+                        }
+                else:
+                    name = ''
+
+            dialog = dialogs.TextEntryDialog(
                     _("New custom playlist name:"),
-                    _("Add To New Playlist..."), name, okbutton=gtk.STOCK_ADD)
+                    _("Add to New Playlist..."), name, okbutton=gtk.STOCK_ADD)
             result = dialog.run()
             if result == gtk.RESPONSE_OK:
                 name = dialog.get_value()
                 if name in self.playlist_manager.playlists:
                     # name is already in use
-                    commondialogs.error(self.parent, _("The "
+                    dialogs.error(self.parent, _("The "
                         "playlist name you entered is already in use."))
                     return
                 elif name == "":
-                    commondialogs.error(self.parent, _("You did "
+                    dialogs.error(self.parent, _("You did "
                         "not enter a name for your playlist"))
                 else:
                     do_add_playlist = True
@@ -461,7 +472,7 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
         self.playlist_target = ("playlist_name", gtk.TARGET_SAME_WIDGET, self.playlist_name_info)
         self.deny_targets = [('',0,0)]
 
-        self.tree = guiutil.DragTreeView(self, True, True)
+        self.tree = PlaylistDragTreeView(self)
         self.tree.connect('row-activated', self.open_item)
         self.tree.set_headers_visible(False)
         self.tree.connect('drag-motion', self.drag_motion)
@@ -524,10 +535,12 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
 
             if item != 'default':
                 menu.connect('append-items', lambda *e:
-                    self.emit('append-items', self.get_selected_tracks()))
+                    self.emit('append-items', self.tree.get_selected_tracks()))
+                menu.connect('replace-items', lambda *e:
+                    self.emit('replace-items', self.tree.get_selected_tracks()))
                 menu.connect('queue-items', lambda *e:
-                    self.emit('queue-items', self.get_selected_tracks()))
-                menu.connect('rating-set', self.set_rating)
+                    self.emit('queue-items', self.tree.get_selected_tracks()))
+                menu.connect('rating-changed', self.on_rating_changed)
 
                 menu.connect('open-playlist', lambda *e:
                     self.open_selected_playlist())
@@ -550,8 +563,8 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
         if settings.get_option('gui/sync_on_tag_change', True) and \
             tag in ['title', 'artist']:
             if self._refresh_id != 0:
-                gobject.source_remove(self._refresh_id)
-            self._refresh_id = gobject.timeout_add(500,
+                glib.source_remove(self._refresh_id)
+            self._refresh_id = glib.timeout_add(500,
                     self._refresh_playlists)
 
     def _refresh_playlists(self):
@@ -560,13 +573,13 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
             need refreshing.
         """
         if settings.get_option('gui/sync_on_tag_change', True):
-            for pl in self.playlist_nodes:
-                self.update_playlist_node(pl)
+            for playlist in self.playlist_nodes:
+                self.update_playlist_node(playlist)
 
-    def refresh_saved_playlist(self, type, object, pl):
+    def refresh_saved_playlist(self, type, object, playlist):
         for plx in self.playlist_nodes:
-            if plx.get_name() == pl.get_name():
-                self.update_playlist_node(pl)
+            if plx.get_name() == playlist.get_name():
+                self.update_playlist_node(playlist)
                 break
 
     def _load_playlists(self):
@@ -616,7 +629,7 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
         """
             Adds a new smart playlist
         """
-        dialog = filtergui.FilterDialog(_('Add Smart Playlist'), self.parent,
+        dialog = FilterDialog(_('Add Smart Playlist'), self.parent,
             CRITERIA)
 
         dialog.set_transient_for(self.parent)
@@ -630,13 +643,13 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
             random = dialog.get_random()
 
             if not name:
-                commondialogs.error(self.parent, _("You did "
+                dialogs.error(self.parent, _("You did "
                     "not enter a name for your playlist"))
                 return
 
             try:
                 pl = self.smart_manager.get_playlist(name)
-                commondialogs.error(self.parent, _("The "
+                dialogs.error(self.parent, _("The "
                     "playlist name you entered is already in use."))
                 return
             except ValueError:
@@ -667,7 +680,7 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
         for k, v in _NMAP.iteritems():
             _REV_NMAP[v] = k
 
-        pl = self.get_selected_playlist(raw=True)
+        pl = self.tree.get_selected_playlist(raw=True)
         if not isinstance(pl, playlist.SmartPlaylist): return
 
         params = pl.search_params
@@ -681,7 +694,7 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
 
         state.reverse()
 
-        dialog = filtergui.FilterDialog(_('Edit Smart Playlist'), self.parent,
+        dialog = FilterDialog(_('Edit Smart Playlist'), self.parent,
             CRITERIA)
 
         dialog.set_transient_for(self.parent)
@@ -703,14 +716,14 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
             random = dialog.get_random()
 
             if not name:
-                commondialogs.error(self.parent, _("You did "
+                dialogs.error(self.parent, _("You did "
                     "not enter a name for your playlist"))
                 return
 
             if not name == pl.name:
                 try:
                     pl = self.smart_manager.get_playlist(name)
-                    commondialogs.error(self.parent, _("The "
+                    dialogs.error(self.parent, _("The "
                         "playlist name you entered is already in use."))
                     return
                 except ValueError:
@@ -742,7 +755,7 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
             # We are being dragged a playlist so
             # we have to reorder them
             playlist_name = selection.get_text()
-            drag_source = self.get_selected_playlist()
+            drag_source = self.tree.get_selected_playlist()
             # verify names
             if drag_source is not None:
                 if drag_source.get_name() == playlist_name:
@@ -800,7 +813,7 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
             # duplicate tracks we have to perform some trickery
             # to make this work properly in all cases
             try:
-                remove_track_index = current_playlist.index(self.get_selected_track())
+                remove_track_index = current_playlist.index(self.tree.get_selected_track())
             except:
                 remove_track_index = None
             if insert_index is not None and remove_track_index is not None:
@@ -867,23 +880,23 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
         """
         #TODO based on info determine what we set in selection_data
         if info == self.playlist_name_info:
-            pl = self.get_selected_playlist()
+            pl = self.tree.get_selected_playlist()
             if pl is not None:
                 selection_data.set(gtk.gdk.SELECTION_TYPE_STRING, 8, pl.get_name())
         else:
-            pl = self.get_selected_playlist()
+            pl = self.tree.get_selected_playlist()
             if pl is not None:
                 tracks = pl.get_tracks()
             else:
-                tracks = [self.get_selected_tracks()]
+                tracks = [self.tree.get_selected_tracks()]
 
             if not tracks: return
 
             for track in tracks:
                 guiutil.DragTreeView.dragged_data[track.get_loc_for_io()] = track
 
-            urls = guiutil.get_urls_for(tracks)
-            selection_data.set_uris(urls)
+            uris = trax.util.get_uris_from_tracks(tracks)
+            selection_data.set_uris(uris)
 
     def drag_motion(self, tv, context, x, y, time):
         """
@@ -903,7 +916,7 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
         # Determine where the drag is coming from
         dragging_playlist = False
         if tv == self.tree:
-            selected_playlist = self.get_selected_playlist()
+            selected_playlist = self.tree.get_selected_playlist()
             if selected_playlist is not None:
                 dragging_playlist = True
 
@@ -956,7 +969,7 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
             @path where we we want it to be saved, with a
                 valid extension we support
         """
-        pl = self.get_selected_playlist()
+        pl = self.tree.get_selected_playlist()
         if pl is not None:
             try:
                 playlist.export_playlist(pl, path)
@@ -965,7 +978,7 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
                 try:
                     playlist.export_playlist(pl, path)
                 except playlist.InvalidPlaylistTypeException:
-                    commondialogs.error(None, _('Invalid file extension, file not saved'))
+                    dialogs.error(None, _('Invalid file extension, file not saved'))
 
     def on_key_released(self, widget, event):
         """
@@ -1049,3 +1062,73 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
             if not iter: break
             self.model.remove(iter)
             iter = self.model.iter_children(node)
+
+class PlaylistDragTreeView(guiutil.DragTreeView):
+    """
+        Custom DragTreeView to retrieve data from playlists
+    """
+    def __init__(self, container, receive=True, source=True):
+        guiutil.DragTreeView.__init__(self, container, receive, source)
+        self.show_cover_drag_icon = False
+
+    def get_selected_tracks(self):
+        """
+            Used by the menu, just basically gets the selected
+            playlist and returns the tracks in it
+        """
+        playlist = self.get_selected_playlist()
+
+        if playlist is not None:
+            return playlist.get_tracks()
+        else:
+            return [self.get_selected_track()]
+
+        return None
+
+    def get_selected_playlist(self, raw=False):
+        """
+            Retrieves the currently selected playlist in
+            the playlists panel.  If a non-playlist is
+            selected it returns None
+
+            @return: the playlist
+        """
+        item = self.get_selected_item(raw=raw)
+
+        if isinstance(item, (playlist.Playlist,
+            playlist.SmartPlaylist)):
+            return item
+        else:
+            return None
+
+    def get_selected_track(self):
+        item = self.get_selected_item()
+
+        if not item: return None
+
+        if isinstance(item, TrackWrapper):
+            return item.track
+        else:
+            return None
+
+    def get_selected_item(self, raw=False):
+        (model, iter) = self.get_selection().get_selected()
+
+        if not iter: return None
+
+        item = model.get_value(iter, 2)
+
+        # for smart playlists
+        if isinstance(item, playlist.SmartPlaylist):
+            if raw: return item
+            return item.get_playlist(self.container.collection)
+        if isinstance(item, radio.RadioItem):
+            if raw: return item
+            return item.get_playlist()
+        elif isinstance(item, playlist.Playlist):
+            return item
+        elif isinstance(item, TrackWrapper):
+            return item
+        else:
+            return None
+
