@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,8 +29,10 @@ import gio
 import gtk
 
 from xl import settings
+from xl.formatter import TrackFormatter
 from xl.nls import gettext as _
-from xlgui import rating
+from xlgui import icons
+from xlgui.widgets import rating
 
 logger = logging.getLogger(__name__)
 
@@ -48,6 +50,7 @@ class Column(object):
     display = ''
     renderer = gtk.CellRendererText
     size = 10 # default size
+    formatter = TrackFormatter('')
 
     def __init__(self, playlist):
         self.playlist = playlist
@@ -61,7 +64,7 @@ class Column(object):
         """
         if not model.iter_is_valid(iter): return
         item = model.get_value(iter, 0)
-        self.playlist.set_cell_weight(cell, item)
+        self.playlist.set_cell_weight(cell, item, iter)
 
     def set_properties(self, col, cellr):
         return
@@ -80,14 +83,11 @@ class TrackNumberColumn(Column):
         """
             Track number
         """
-        item = model.get_value(iter, 0)
+        track = model.get_value(iter, 0)
 
-        track = item.get_tag_display("tracknumber")
-        if track is None:
-            cell.set_property('text', '')
-        else:
-            cell.set_property('text', track)
-        self.playlist.set_cell_weight(cell, item)
+        self.formatter.set_property('format', '$%s' % self.id)
+        cell.set_property('text', self.formatter.format(track))
+        self.playlist.set_cell_weight(cell, track, iter)
 
     def set_properties(self, col, cellr):
         cellr.set_property('xalign', 1.0)
@@ -121,16 +121,11 @@ class LengthColumn(Column):
         """
             Formats the track length
         """
-        item = model.get_value(iter, 0)
-        try:
-            seconds = item.get_tag_raw("__length")
-            text = _("%(minutes)d:%(seconds)02d") % \
-                {'minutes' : seconds // 60, 'seconds' : seconds % 60}
-        except:
-            #TRANSLATORS: Default track length
-            text = _("0:00")
-        cell.set_property('text', text)
-        self.playlist.set_cell_weight(cell, item)
+        track = model.get_value(iter, 0)
+
+        self.formatter.set_property('format', '$%s' % self.id)
+        cell.set_property('text', self.formatter.format(track))
+        self.playlist.set_cell_weight(cell, track, iter)
 
     def set_properties(self, col, cellr):
         cellr.set_property('xalign', 1.0)
@@ -144,41 +139,29 @@ class DiscNumberColumn(Column):
         """
             Disc number
         """
-        item = model.get_value(iter, 0)
+        track = model.get_value(iter, 0)
 
-        disc = item.get_tag_display("discnumber")
+        disc = track.get_tag_display("discnumber")
         if disc is None:
             cell.set_property('text', '')
         else:
             cell.set_property('text', disc)
-        self.playlist.set_cell_weight(cell, item)
+        self.playlist.set_cell_weight(cell, track, iter)
 
     def set_properties(self, col, cellr):
         cellr.set_property('xalign', 1.0)
 
 class RatingColumn(Column):
-    steps = settings.get_option('miscellaneous/rating_steps', 5)
-    size = 12 * steps
     display = _('Rating')
-    renderer = gtk.CellRendererPixbuf
+    renderer = rating.RatingCellRenderer
     id = '__rating'
 
     def data_func(self, col, cell, model, iter):
-        item = model.get_value(iter, 0)
-        try:
-            idx = item.get_rating()
-            cell.set_property('pixbuf',
-                rating.rating_images[idx])
-        except IndexError:
-            logger.debug("idx_error")
-            if idx > steps: idx = steps
-            elif idx < 0: idx = 0
-            cell.set_property('pixbuf',
-                rating.rating_images[idx])
+        track = model.get_value(iter, 0)
+        cell.props.rating = track.get_rating()
 
     def set_properties(self, col, cellr):
         cellr.set_property('follow-state', False)
-        col.set_attributes(cellr, pixbuf=1)
 
 class DateColumn(Column):
     size = 50
@@ -199,9 +182,9 @@ class BitrateColumn(Column):
         """
             Shows the bitrate
         """
-        item = model.get_value(iter, 0)
-        cell.set_property('text', item.get_tag_display("__bitrate"))
-        self.playlist.set_cell_weight(cell, item)
+        track = model.get_value(iter, 0)
+        cell.set_property('text', track.get_tag_display("__bitrate"))
+        self.playlist.set_cell_weight(cell, track, iter)
 
     def set_properties(self, col, cellr):
         cellr.set_property('xalign', 1.0)
@@ -217,10 +200,11 @@ class FilenameColumn(Column):
     id = 'filename'
 
     def data_func(self, col, cell, model, iter):
-        item = model.get_value(iter, 0)
-        cell.set_property('text',
-                gio.File(item.get_loc_for_io()).get_basename())
-        self.playlist.set_cell_weight(cell, item)
+        track = model.get_value(iter, 0)
+
+        self.formatter.set_property('format', '$%s' % self.id)
+        cell.set_property('text', self.formatter.format(track))
+        self.playlist.set_cell_weight(cell, track)
 
 class PlayCountColumn(Column):
     size = 50
@@ -247,33 +231,11 @@ class LastPlayedColumn(Column):
         """
             Formats the last played time string
         """
-        item = model.get_value(iter, 0)
-        #TRANSLATORS: Time strings for today, yesterday, default
-        try:
-            if item.get_tag_raw('__last_played') is None:
-                text = _("Never")
-            else:
-                import time
-                ct = time.time()
-                now = time.localtime(ct)
-                yday = time.localtime(ct - 86400)
-                ydaytime = time.mktime((yday.tm_year, yday.tm_mon, yday.tm_mday, \
-                    0, 0, 0, yday.tm_wday, yday.tm_yday, yday.tm_isdst))
-                lptime = time.localtime(item.get_tag_raw('__last_played'))
-                if now.tm_year == lptime.tm_year and \
-                   now.tm_mon == lptime.tm_mon and \
-                   now.tm_mday == lptime.tm_mday:
-                    text = _("Today")
-                elif ydaytime <= item.get_tag_raw('__last_played'):
-                    text = _("Yesterday")
-                else:
-                    text = _("%(year)d-%(month)02d-%(day)02d") % \
-                    {'year' : lptime.tm_year , 'month' : lptime.tm_mon, \
-                     'day' : lptime.tm_mday}
-        except:
-            text = _("Never")
-        cell.set_property('text', text)
-        self.playlist.set_cell_weight(cell, item)
+        track = model.get_value(iter, 0)
+
+        self.formatter.set_property('format', '$%s' % self.id)
+        cell.set_property('text', self.formatter.format(track))
+        self.playlist.set_cell_weight(cell, track, iter)
 
 # this is where everything gets set up, including the menu items
 COLUMNS = {}
