@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Mathias Brodala
+# Copyright (C) 2009-2010 Mathias Brodala
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,17 +14,40 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-import gobject, gtk, os
-import minimodeprefs, mmwidgets
+import cairo
+import gobject
+import gtk
+import os
+
 from xl import event, plugins, settings, xdg
+from xl.formatter import TrackFormatter
 from xl.nls import gettext as _
+from xlgui import icons
+
+import minimode_preferences, mmwidgets
 
 MINIMODE = None
 
+def __migrate_fixed_controls():
+    """
+        Makes sure fixed controls are selected,
+        mostly for migration from older versions
+    """
+    option_name = 'plugin/minimode/selected_controls'
+
+    if settings.MANAGER.has_option(option_name):
+        selected_controls = settings.get_option(option_name)
+
+        if not 'restore' in selected_controls:
+            selected_controls += ['restore']
+            settings.set_option(option_name, selected_controls)
+
 def enable(exaile):
     """
         Enables the mini mode plugin
     """
+    __migrate_fixed_controls()
+
     if exaile.loading:
         event.add_callback(_enable, 'gui_loaded')
     else:
@@ -42,12 +65,11 @@ def disable(exaile):
         Disables the mini mode plugin
     """
     global MINIMODE
-    if MINIMODE:
-        MINIMODE.destroy()
-        MINIMODE = None
+    MINIMODE.destroy()
+    MINIMODE = None
 
-def get_prefs_pane():
-    return minimodeprefs
+def get_preferences_pane():
+    return minimode_preferences
 
 class MiniMode(gtk.Window):
     """
@@ -58,39 +80,27 @@ class MiniMode(gtk.Window):
             Sets up the mini mode main window and
             options to access it
         """
+        gtk.Window.__init__(self)
+        self.set_title('Exaile')
+        self.set_resizable(False)
+
         self.exaile = exaile
         self._active = False
         self.defaults = self.exaile.plugins.get_plugin_default_preferences('minimode')
         self.defaults['plugin/minimode/horizontal_position'] = 10
         self.defaults['plugin/minimode/vertical_position'] = 10
 
-        controlpref = minimodeprefs.SelectedControlsPreference
-        self.available_controls = controlpref.available_items.keys() or []
-        self.fixed_items = controlpref.fixed_items.keys() or []
-
-        gtk.Window.__init__(self)
-        self.set_title('Exaile')
-        self.set_resizable(False)
-
-        self.formatter = mmwidgets.TrackFormatter(
+        self.formatter = TrackFormatter(
             self.get_option('plugin/minimode/track_title_format')
         )
 
         self.box = mmwidgets.WidgetBox(spacing=3)
-        frame = gtk.Frame()
-        frame.set_shadow_type(gtk.SHADOW_OUT)
-        frame.add(self.box)
-        self.add(frame)
-
-        self.register_widgets()
-        controls = self.get_option('plugin/minimode/selected_controls')
-        controls += self.fixed_items
-        self.update_widgets(controls)
-
-        self.update_position()
+        self.border_frame = gtk.Frame()
+        self.border_frame.add(self.box)
+        self.add(self.border_frame)
 
         basedir = os.path.dirname(os.path.abspath(__file__))
-        self.exaile.gui.icons.add_stock_from_directory('exaile-minimode',
+        icons.MANAGER.add_stock_from_directory('exaile-minimode',
             os.path.join(basedir, 'icons'))
 
         self.menuitem = mmwidgets.MenuItem(self.on_menuitem_activate)
@@ -104,10 +114,16 @@ class MiniMode(gtk.Window):
         self.exaile.gui.main.window.add_accel_group(self.accel_group)
         self.add_accel_group(self.accel_group)
 
+        self.register_widgets()
+        self.update_widgets(self.get_option('plugin/minimode/selected_controls'))
+        self.update_position()
+
         self._configure_id = None
         self._main_visible_toggle_id = None
 
         self.connect('show', self.on_show)
+        self.connect('expose-event', self.on_expose_event)
+        self.connect('screen-changed', self.on_screen_changed)
         self.connect('delete-event', self.on_delete_event)
         self.exaile.gui.main.connect('main-visible-toggle',
             self.on_main_visible_toggle)
@@ -160,7 +176,7 @@ class MiniMode(gtk.Window):
         self.show_all()
         if self._configure_id is None:
             self._configure_id = self.connect('configure-event',
-                self.on_configure)
+                self.on_configure_event)
 
     def toggle_visible(self):
         """
@@ -187,16 +203,35 @@ class MiniMode(gtk.Window):
                 if value: self.stick()
                 else: self.unstick()
             elif option == 'plugin/minimode/display_window_decorations':
-                self.set_decorated(value)
+                if value:
+                    decoration_type = self.get_option('plugin/minimode/window_decoration_type')
+
+                    if decoration_type == 'full':
+                        self.set_decorated(True)
+                        self.border_frame.set_shadow_type(gtk.SHADOW_NONE)
+                    elif decoration_type == 'simple':
+                        self.set_decorated(False)
+                        self.border_frame.set_shadow_type(gtk.SHADOW_OUT)
+                else:
+                    self.set_decorated(False)
+                    self.border_frame.set_shadow_type(gtk.SHADOW_NONE)
+            elif option == 'plugin/minimode/use_alpha':
+                self.unrealize()
+
+                screen = self.get_screen()
+                colormap = screen.get_rgba_colormap() or screen.get_rgb_colormap()
+                self.set_colormap(colormap)
+                self.set_app_paintable(value)
+
+                self.realize()
             elif option == 'plugin/minimode/horizontal_position':
                 self.update_position()
             elif option == 'plugin/minimode/vertical_position':
                 self.update_position()
             elif option == 'plugin/minimode/selected_controls':
-                value += self.fixed_items
                 self.update_widgets(value)
             elif option == 'plugin/minimode/track_title_format':
-                self.formatter.set_format(value)
+                self.formatter.set_property('format', value)
 
     def update_position(self):
         """
@@ -213,27 +248,31 @@ class MiniMode(gtk.Window):
         """
         widgets = {
             'previous': (mmwidgets.Button,
-                ['gtk-media-previous', _('Previous Track'), self.on_previous]),
+                [gtk.STOCK_MEDIA_PREVIOUS, _('Previous Track'),
+                    self.on_previous_clicked]),
             'next': (mmwidgets.Button,
-                ['gtk-media-next', _('Next Track'), self.on_next]),
+                [gtk.STOCK_MEDIA_NEXT, _('Next Track'),
+                    self.on_next_clicked]),
             'play_pause': (mmwidgets.PlayPauseButton,
-                [self.exaile.player, self.on_play_pause]),
-            'stop': (mmwidgets.Button,
-                ['gtk-media-stop', _('Stop Playback'), self.on_stop]),
+                [self.exaile.player, self.on_play_pause_clicked]),
+            'stop': (mmwidgets.StopButton,
+                [self.exaile.player, self.exaile.queue]),
             'volume': (mmwidgets.VolumeButton,
                 [self.exaile.player, self.on_volume_changed]),
-            'restore': (mmwidgets.Button,
-                ['gtk-fullscreen', _('Restore Main Window'), self.on_restore]),
+            'restore': (mmwidgets.RestoreButton,
+                [self.accel_group, self.on_restore_clicked]),
             'progress_bar': (mmwidgets.ProgressBar,
-                [self.exaile.player, self.on_track_seeked]),
+                [self.exaile.player, self.on_seeked]),
             'track_selector': (mmwidgets.TrackSelector,
                 [self.exaile.gui.main, self.exaile.queue, self.formatter,
                  self.on_track_change]),
             'playlist_button': (mmwidgets.PlaylistButton,
-                [self.exaile.gui.main, self.exaile.queue, self.formatter,
-                 self.on_track_change])
+                [self.exaile.gui.main, self.exaile.player, self.exaile.queue,
+                 self.formatter, self.on_track_change]),
+            'progress_button': (mmwidgets.ProgressButton,
+                [self.on_track_change, self.on_seeked]),
+            'rating': (mmwidgets.RatingWidget, [])
         }
-        # TODO: PlaylistProgressBar
 
         self.box.register_widgets(widgets)
 
@@ -265,19 +304,19 @@ class MiniMode(gtk.Window):
         self.toggle_visible()
         self._active = not self._active
 
-    def on_previous(self, button):
+    def on_previous_clicked(self, button):
         """
             Jumps to the previous track
         """
         self.exaile.queue.prev()
 
-    def on_next(self, button):
+    def on_next_clicked(self, button):
         """
             Jumps to the next track
         """
         self.exaile.queue.next()
 
-    def on_play_pause(self, button):
+    def on_play_pause_clicked(self, button):
         """
             Toggles between playback and pause mode
         """
@@ -286,13 +325,7 @@ class MiniMode(gtk.Window):
         else:
             self.exaile.queue.play()
 
-    def on_stop(self, button):
-        """
-            Stops playback
-        """
-        self.exaile.player.stop()
-
-    def on_restore(self, button):
+    def on_restore_clicked(self, button):
         """
             Hides mini mode on button click
         """
@@ -308,12 +341,11 @@ class MiniMode(gtk.Window):
             self.exaile.queue.current_playlist.set_current_pos(index)
             self.exaile.queue.play(track)
 
-    def on_track_seeked(self, progress_bar, position):
+    def on_seeked(self, progress_bar, position):
         """
             Handles seeking in the progress bar
         """
-        duration = self.exaile.player.current.get_duration()
-        self.exaile.player.seek(duration * float(position))
+        self.exaile.player.set_progress(position)
 
     def on_volume_changed(self, volume_button, value):
         """
@@ -333,7 +365,7 @@ class MiniMode(gtk.Window):
             return True
         return False
 
-    def on_configure(self, widget, event):
+    def on_configure_event(self, widget, event):
         """
             Handles movement of the window
         """
@@ -348,6 +380,36 @@ class MiniMode(gtk.Window):
         self.resize(*self.size_request())
         self.queue_draw()
 
+    def on_expose_event(self, widget, event):
+        """
+            Paints the window alpha transparency
+        """
+        context = widget.window.cairo_create()
+
+        context.rectangle(event.area.x, event.area.y,
+            event.area.width, event.area.height)
+        context.clip()
+
+        background = widget.style.bg[gtk.STATE_NORMAL]
+        opacity = 1 - self.get_option('plugin/minimode/transparency')
+        context.set_source_rgba(
+            float(background.red) / 256**2,
+            float(background.green) / 256**2,
+            float(background.blue) / 256**2,
+            opacity
+        )
+        context.set_operator(cairo.OPERATOR_SOURCE)
+
+        context.paint()
+
+    def on_screen_changed(self, widget, event):
+        """
+            Updates the colormap on screen change
+        """
+        screen = widget.get_screen()
+        colormap = screen.get_rgba_colormap() or screen.get_rgb_colormap()
+        self.window.set_colormap(rgbamap)
+
     def on_delete_event(self, widget, event):
         """
             Closes application on quit
@@ -356,3 +418,4 @@ class MiniMode(gtk.Window):
         self.exaile.gui.main.quit()
         return True
 
+# vim: et sts=4 sw=4
